/**
 * The MySensors Arduino library handles the wireless radio link and protocol
 * between your home built sensors/actuators and HA controller of choice.
 * The sensors forms a self healing radio network with optional repeaters. Each
 * repeater and gateway builds a routing tables in EEPROM which keeps track of the
 * network topology allowing messages to be routed to nodes.
 *
 * Created by Henrik Ekblad <henrik.ekblad@mysensors.org>
 * Copyright (C) 2013-2015 Sensnology AB
 * Full contributor list: https://github.com/mysensors/Arduino/graphs/contributors
 *
 * Documentation: http://www.mysensors.org
 * Support Forum: http://forum.mysensors.org
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 *
 *******************************
 *
 * REVISION HISTORY
 * Version 1.0 - Henrik Ekblad
 * 
 * DESCRIPTION
 * Example sketch showing how to control physical relays. 
 * This example will remember relay state after power failure.
 * http://www.mysensors.org/build/relay
 * 
 * 
 * Dzairo add .
 * Add I2C PCF8574 for 32 relays.
 * Define V_TEMP to save state or relay to eeprom or not .
 * if Set S_Temp to 1 then relay state is writed to eeprom , after reset or power up 
 * relay read this state and switch .
 * if Clear S_Temp to 0 then clear memory relay state and start with all relay close .
 * 
 */ 

#include <MySigningNone.h>
#include <MyTransportNRF24.h>
#include <MyTransportRFM69.h>
#include <MyHwATMega328.h>
#include <MySensor.h>
#include <SPI.h>
#include <Wire.h> 

#define RELAY_0  0  // I2C expander
#define NUMBER_OF_RELAYS 32 // Total number of attached relays
#define RELAY_ON 1  // GPIO value to write to turn on attached relay
#define RELAY_OFF 0 // GPIO value to write to turn off attached relay

#define SAVESTATE NUMBER_OF_RELAYS // How many save state we have
#define SYES 1   // If we want save state of relay to eeprom
#define SNO 0    // If we don't want save state of relay to eeprom

// NRFRF24L01 radio driver (set low transmit power by default) 
MyTransportNRF24 radio(RF24_CE_PIN, RF24_CS_PIN, RF24_PA_LEVEL_GW);  
//MyTransportRFM69 radio;
// Message signing driver (none default)
//MySigningNone signer;
// Select AtMega328 hardware profile
MyHwATMega328 hw;
// Construct MySensors library
MySensor gw(radio, hw);
// Relay variable for 4x 8bit I2C relay.
unsigned long Relay=0;
byte SaveState=0;
void setup()  
{   
  // I2C beggin
  Wire.begin(); 
  // Initialize library and add callback for incoming messages
  gw.begin(incomingMessage, 58, true);//AUTO
  // Send the sketch version information to the gateway and Controller
  gw.sendSketchInfo("RelayI2C", "1.0");
  // Present Save state , 0 - binary 
  gw.present(SAVESTATE, S_TEMP);
  // Load state to variable
  SaveState=gw.loadState(SAVESTATE);
  // Start position
  Relay=0;
  // Fetch relay status
  for (int sensor=0, pin=RELAY_0; sensor<=NUMBER_OF_RELAYS-1;sensor++, pin++) {
    // Register all sensors to gw (they will be created as child devices)
    gw.present(sensor, S_LIGHT);
    
    // If not set save state then clear eeprom last state if relay..
    if (!SaveState)gw.saveState(pin, 0);else 
    // Set relay to last known state (using eeprom storage) 
      bitWrite(Relay,pin,gw.loadState(sensor));
  }
    Wire.beginTransmission(32);     //Begin the transmission to PCF8574
    Wire.write((byte)255-(Relay & 0x000000ffUL));                //Send the data to PCF8574 0..7
    Wire.endTransmission();         //End the Transmission 

    Wire.beginTransmission(33);     //Begin the transmission to PCF8574
    Wire.write((byte)255-(Relay & 0x0000ff00UL)>> 8);            //Send the data to PCF8574 0..7
    Wire.endTransmission();         //End the Transmission 

    Wire.beginTransmission(34);     //Begin the transmission to PCF8574
    Wire.write((byte)255-(Relay & 0x00ff0000UL)>> 16);           //Send the data to PCF8574 0..7
    Wire.endTransmission();         //End the Transmission 

    Wire.beginTransmission(35);     //Begin the transmission to PCF8574
    Wire.write((byte)255-(Relay & 0xff000000UL)>> 24);           //Send the data to PCF8574 0..7
    Wire.endTransmission();         //End the Transmission 
  
}

void loop() 
{
  // Alway process incoming messages whenever possible
  gw.process();
}

void incomingMessage(const MyMessage &message) {
  // We only expect one type of message from controller. But we better check anyway.
  if (message.type==V_LIGHT) {
     // Change relay state
    if (bitRead(Relay,message.sensor)) bitWrite(Relay,message.sensor,RELAY_OFF); else bitWrite(Relay,message.sensor,RELAY_ON);
    // Send data to I2C expander .. 
    Wire.beginTransmission(32);     //Begin the transmission to PCF8574
    Wire.write((byte)255-(Relay & 0x000000ffUL));                //Send the data to PCF8574 0..7
    Wire.endTransmission();         //End the Transmission 

    Wire.beginTransmission(33);     //Begin the transmission to PCF8574
    Wire.write((byte)255-(Relay & 0x0000ff00UL)>> 8);            //Send the data to PCF8574 0..7
    Wire.endTransmission();         //End the Transmission 

    Wire.beginTransmission(34);     //Begin the transmission to PCF8574
    Wire.write((byte)255-(Relay & 0x00ff0000UL)>> 16);           //Send the data to PCF8574 0..7
    Wire.endTransmission();         //End the Transmission 

    Wire.beginTransmission(35);     //Begin the transmission to PCF8574
    Wire.write((byte)255-(Relay & 0xff000000UL)>> 24);           //Send the data to PCF8574 0..7
    Wire.endTransmission();         //End the Transmission 

     // Store state in eeprom
    if (SaveState) {if (bitRead(Relay,message.sensor)) gw.saveState(message.sensor, 1); else gw.saveState(message.sensor, 0);}
     // Write some debug info
     Serial.print("Incoming change for sensor:");
     Serial.print(message.sensor);
     Serial.print(", New status: ");
     Serial.println(message.getBool());
   } 

  if (message.type==V_TEMP) {
  gw.saveState(message.sensor, message.getBool());
  SaveState= message.getBool(); 
  }
   
}

